import 'reflect-metadata'
import express from "express"
import {param, query} from "express-validator"
import {Container} from "typedi"
import {TextbookModel} from "../models/textbook-model"
import expressAsyncHandler from "express-async-handler"
import {TextbookService} from "../services/textbook-service"
import {hasValidationErrors} from "../includes"

let textbookRouter = express.Router()
let textbookModel = Container.get(TextbookModel)
let textbookService = Container.get(TextbookService)
textbookRouter.get('/',
    expressAsyncHandler(async (req: express.Request, res: express.Response) => {
        res.json(await textbookService.listTextbooks())
    })
)
textbookRouter.get('/:textbook/:section?/:exercise?',
    param('textbook').notEmpty().bail().custom(input => {
        if (!textbookModel.existTextbook(input)) {
            throw new Error('textbook not exist')
        }
        return true
    }),
    query('page').optional().isInt({min: 1}),
    param('section').optional().notEmpty().bail().custom((input, {req}) => {
        if (!textbookModel.existSection(req.params!.textbook, input.replace(/-/g, '/'))) {
            throw new Error('section not exist')
        }
        return true
    }),
    param('exercise').optional().notEmpty().bail().custom((input, {req}) => {
        if (!textbookModel.existExercise(req.params!.textbook, req.params!.section.replace(/-/g, '/'), input)) {
            throw new Error('exercise not exist')
        }
        return true
    }),
    expressAsyncHandler(async (req: express.Request, res: express.Response) => {
        if (hasValidationErrors(req, res)) return
        if (req.query.page) {
            res.json(await textbookService.getPage(req.params.textbook, Number(req.query.page)))
        } else if (req.params.exercise) {
            res.json(await textbookService.getExercise(req.params.textbook, req.params.section.replace(/-/g, '/'), req.params.exercise))
        } else if (req.params.section) {
            res.json(await textbookService.getSection(req.params.textbook, req.params.section.replace(/-/g, '/')))
        } else {
            res.json(await textbookService.getTextbook(req.params.textbook))
        }
    })
)


export {textbookRouter}
